# #
# $Date: 1996/12/12 10:48:51 $ $Author: frankp $ $Revision: 1.40.2.2 $ #

# #
# frankp, 03/11/94 #

#++
  SqMatrix.mu

	Dom::SquareMatrix -- the domains of square matrices

	Dom::SquareMatrix(n,[R])

	n - dimension (positive integer)
	R - (optional) the coefficient domain (a Cat::Rng)

	A domain Dom::SquareMatrix(n,R) created by Dom::SquareMatrix 
	represents the Cat::Rng of n times n matrices with coefficients 
	from a Cat::Rng R.

	If 'R' is missing then the coefficient domain will be the
	domain 'Dom::ExpressionField()'.

	An element of Dom::SquareMatrix(n,R) has the same representation as
	an element of the domain Dom::Matrix(R). This ensures that all 
	methods for Dom::Matrix(R) may be used by Dom::SquareMatrix(n,R).

	Note that the results of the methods "delRow", "delCol", 
	"stackMatrix", "concatMatrix", "row" and "col" will be
	elements of the super domain Dom::Matrix(R).
 
	The method "newThis" (inherited from Dom::Matrix(R)) must be used
	with coution because this method does not tests its arguments
	(thus it is possible to create matrices which are non-square,
	for example).

	Entries: one, zero
	Methods: random
++#

Dom::SquareMatrix:= DomainConstructor(

# name #
    Dom::SquareMatrix,
# arguments #
    [ n, R ],
# local variables #
    [ Rzero, Rone ],
# initialisation #
    (if args(0) < 1 or args(0) > 2 then error("wrong no of args") end_if;
     if not testtype( n,Type::PosInt ) then
	error("dimension must be a positive integer")
     end_if;
     if args(0) = 1 then R:= Dom::ExpressionField()
     elif not (R::hasProp( Cat::Rng ) = TRUE) then
	error("coefficients must be from a Cat::Rng")
     end_if;
     Rone:= R::one;
     Rzero:= R::zero
    ),
# super-domains #
    Dom::Matrix(R),
# categories #
    [ Cat::SquareMatrixCat(R) ],
# axioms #
    [ (if R::hasProp( Ax::canonicalRep ) then Ax::canonicalRep end_if) ],
# #
    "new" = proc(f)
	local r,c,x;
    begin
	case args(0)
	of 0 do
	    return( this::zero )
	of 1 do
	    if contains( {DOM_EXEC, DOM_PROC}, domtype(f) ) then
		return( extsubsop( Dom::Matrix(R)(n,n,f),0=this ) )
	    else
	        x:= this::convert(f);
	        if x = FAIL then
		    error("invalid argument")
	        else
		    return( x )
	        end_if
	    end_if
	of 2 do	
	    r:= args(1); c:= args(2);
	    if testtype( r,Type::PosInt ) and testtype( c,Type::PosInt ) then
		if r = n and c = n then return( this::zero )
		else error("invalid dimension")
		end_if
	    elif c = hold(Diagonal) then
		return( extsubsop( Dom::Matrix(R)(n,n,r,c),0=this ) )
	    else
		error("invalid arguments")
	    end_if
	of 3 do
	of 4 do
	    x:= extsubsop( Dom::Matrix(R)(args()),0=this );
	    if [extop(x,1),extop(x,2)] <> [n,n] then
		error("invalid dimension")
	    else
		return( x )
	    end_if
	otherwise
	    error("wrong no of arguments")
	end_case
    end_proc,
# #
    "convert" = proc(e)
	local t;
    begin
	if domtype(e) = this then return( e ) end_if;
	t := domattr(Dom::Matrix(R),"convert")( e );
	if t = FAIL then return( FAIL ) end_if;
	if t::dimen(t) <> [n,n] then
	    return( FAIL )
	else
	    extsubsop( t,0=this )
	end_if
    end_proc,
# #
    "one" = (if Rone <> FAIL then
	new( this,n,n,
	  array(1..n,1..n,[[Rzero $ i-1,Rone,Rzero $ n-i] $ hold(i)=1..n])
	   )
    end_if),
# #
    "zero" = new( this,n,n,
	array( 1..n,1..n,[ [Rzero $ n] $ n ] ) ),
# #
    "dimen" = [n,n],
# #
    "delRow" = domattr(Dom::Matrix(R),"delRow"),
# #
    "delCol" = domattr(Dom::Matrix(R),"delCol"),
# #
    "stackMatrix" = domattr(Dom::Matrix(R),"stackMatrix"),
# #
    "concatMatrix" = domattr(Dom::Matrix(R),"concatMatrix"),
# #
    "row" = domattr(Dom::Matrix(R),"row"),
# #
    "col" = domattr(Dom::Matrix(R),"col"),
# #
    "randomDimen" = [n,n],
# #
    "random" = (if R::random <> FAIL then
    # returns a n times n random matrix #
            fun( new( this,n,n,array( 1..n,1..n,
                [[R::random() $ hold(j)=1..n] $ hold(i)=1..n]
            )) )
    end_if)

):

# end of file #
